package com.tarento.training.services;

import com.tarento.training.models.GeocodingResponse;
import io.github.resilience4j.circuitbreaker.annotation.CircuitBreaker;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

@Service
@Slf4j
public class GeocodingService {
    @Autowired
    private RestTemplate restTemplate;

    @Value("${rapidapi.geocoding.url}")
    private String geocodingUrl;

    @Value("${rapidapi.geocoding.key}")
    private String apiKey;

    @Value("${rapidapi.geocoding.host}")
    private String apiHost;

    @CircuitBreaker(name = "geocodingService", fallbackMethod = "fallbackGetAddressInfo")
    public GeocodingResponse getAddressInfo(double latitude, double longitude) {
        log.info("Getting address info...");
        String url = geocodingUrl + "?latlng=" + latitude + "," + longitude;

        RestTemplate restTemplate = new RestTemplate();
        restTemplate.getInterceptors().add((request, body, execution) -> {
            request.getHeaders().set("x-rapidapi-host", apiHost);
            request.getHeaders().set("x-rapidapi-key", apiKey);
            return execution.execute(request, body);
        });

        return restTemplate.getForObject(url, GeocodingResponse.class);
    }

    public GeocodingResponse fallbackGetAddressInfo(double latitude, double longitude, Throwable throwable) {
            GeocodingResponse fallbackResponse = new GeocodingResponse();
        log.info("Getting address info from fallback...");
        fallbackResponse.setStatus("FALLBACK");
        return fallbackResponse;
    }
}
